@@ -1,5 +1,4 @@
-/* $Id: uio.h 4022 2008-03-31 06:11:07Z rra $
- *
+/*
  * Portability wrapper around <sys/uio.h>.
  *
  * Provides a definition of the iovec struct for platforms that don't have it
@@ -15,9 +14,10 @@
 
 #include <sys/types.h>
 
-#ifdef HAVE_SYS_UIO_H
+/* remctl.h provides its own definition of this struct on Windows. */
+#if defined(HAVE_SYS_UIO_H)
 # include <sys/uio.h>
-#else
+#elif !defined(REMCTL_H)
 struct iovec {
     void *iov_base;
     size_t iov_len;
