@@ -6,7 +6,7 @@
  *
  * Written by Russ Allbery <rra@stanford.edu>
  * Based on work by Anton Ushakov
- * Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008
+ * Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
  *     Board of Trustees, Leland Stanford Jr. University
  *
  * See LICENSE for licensing terms.
@@ -14,6 +14,7 @@
 
 #include <config.h>
 #include <portable/system.h>
+#include <portable/uio.h>
 
 #include <errno.h>
 #include <fcntl.h>
@@ -30,16 +31,18 @@
 struct process {
     bool reaped;                /* Whether we've reaped the process. */
     int fds[2];                 /* Array of file descriptors for output. */
+    int stdin_fd;               /* File descriptor for standard input. */
+    struct iovec *input;        /* Data to pass on standard input. */
     pid_t pid;                  /* Process ID of child. */
     int status;                 /* Exit status. */
 };
 
 
 /*
- * Processes the output from an external program.  Takes the client struct, an
- * array of file descriptors representing the output streams from the client,
- * and a count of streams.  Reads from all the streams as output is available,
- * stopping when they all reach EOF.
+ * Processes the input to and output from an external program.  Takes the
+ * client struct and a struct representing the running process.  Feeds input
+ * data to the process on standard input and reads from all the streams as
+ * output is available, stopping when they all reach EOF.
  *
  * For protocol v2 and higher, we can send the output immediately as we get
  * it.  For protocol v1, we instead accumulate the output in the buffer stored
@@ -53,10 +56,11 @@ server_process_output(struct client *client, struct process *process)
 {
     char junk[BUFSIZ];
     char *p;
+    size_t offset = 0;
     size_t left = MAXBUFFER;
-    ssize_t status[2];
+    ssize_t status[2], instatus;
     int i, maxfd, fd, result;
-    fd_set fdset;
+    fd_set readfds, writefds;
     struct timeval timeout;
 
     /* If we haven't allocated an output buffer, do so now. */
@@ -64,9 +68,14 @@ server_process_output(struct client *client, struct process *process)
         client->output = xmalloc(MAXBUFFER);
     p = client->output;
 
-    /* Initialize read status for standard output and standard error. */
+    /*
+     * Initialize read status for standard output and standard error and write
+     * status for standard input to the process.  Non-zero says that we keep
+     * trying to read or write.
+     */
     status[0] = -1;
     status[1] = -1;
+    instatus = (process->input != NULL ? -1 : 0);
 
     /*
      * Now, loop while we have input.  We no longer have input if the return
@@ -81,17 +90,29 @@ server_process_output(struct client *client, struct process *process)
      * child process aren't closed yet.  To catch this case, call waitpid with
      * the WNOHANG flag each time through the select loop and decide we're
      * done as soon as our child has exited.
+     *
+     * Meanwhile, if we have input data, then as long as we've not gotten an
+     * EPIPE error from sending input data to the process we keep writing
+     * input data as select indicates the process can receive it.  However, we
+     * don't care if we've sent all input data before the process says it's
+     * done and exits.
      */
     while (!process->reaped) {
-        FD_ZERO(&fdset);
+        FD_ZERO(&readfds);
         maxfd = -1;
         for (i = 0; i < 2; i++) {
             if (status[i] != 0) {
-                FD_SET(process->fds[i], &fdset);
                 if (process->fds[i] > maxfd)
                     maxfd = process->fds[i];
+                FD_SET(process->fds[i], &readfds);
             }
         }
+        if (instatus != 0) {
+            FD_ZERO(&writefds);
+            if (process->stdin_fd > maxfd)
+                maxfd = process->stdin_fd;
+            FD_SET(process->stdin_fd, &writefds);
+        }
         if (maxfd == -1)
             break;
 
@@ -125,13 +146,41 @@ server_process_output(struct client *client, struct process *process)
             process->reaped = true;
             timeout.tv_sec = 0;
         }
-        result = select(maxfd + 1, &fdset, NULL, NULL, &timeout);
+        if (instatus != 0)
+            result = select(maxfd + 1, &readfds, &writefds, NULL, &timeout);
+        else
+            result = select(maxfd + 1, &readfds, NULL, NULL, &timeout);
         if (result < 0 && errno != EINTR) {
             syswarn("select failed");
             server_send_error(client, ERROR_INTERNAL, "Internal failure");
             goto fail;
         }
 
+        /*
+         * If we can still write and our child selected for writing, send as
+         * much data as we can.
+         */
+        if (instatus != 0 && FD_ISSET(process->stdin_fd, &writefds)) {
+            instatus = write(process->stdin_fd,
+                             (char *) process->input->iov_base + offset,
+                             process->input->iov_len - offset);
+            if (instatus < 0) {
+                if (errno == EPIPE)
+                    instatus = 0;
+                else if (errno != EINTR && errno != EAGAIN) {
+                    syswarn("write failed");
+                    server_send_error(client, ERROR_INTERNAL,
+                                      "Internal failure");
+                    goto fail;
+                }
+            }
+            offset += instatus;
+            if (offset >= process->input->iov_len) {
+                close(process->stdin_fd);
+                instatus = 0;
+            }
+        }
+
         /*
          * Iterate through each set file descriptor and read its output.  If
          * we're using protocol version one, we append all the output together
@@ -140,7 +189,7 @@ server_process_output(struct client *client, struct process *process)
          */
         for (i = 0; i < 2; i++) {
             fd = process->fds[i];
-            if (!FD_ISSET(fd, &fdset))
+            if (!FD_ISSET(fd, &readfds))
                 continue;
             if (client->protocol == 1) {
                 if (left > 0) {
@@ -187,12 +236,12 @@ fail:
  * executable or any error message.  Returns 0 on success and a negative
  * integer on failure.
  *
- * Using the type and the service, the following argument, a lookup in the
- * conf data structure is done to find the command executable and acl file.
- * If the conf file, and subsequently the conf data structure contains an
- * entry for this type with service equal to "ALL", that is a wildcard match
- * for any given service.  The first argument is then replaced with the actual
- * program name to be executed.
+ * Using the command and the subcommand, the following argument, a lookup in
+ * the conf data structure is done to find the command executable and acl
+ * file.  If the conf file, and subsequently the conf data structure contains
+ * an entry for this command with subcommand equal to "ALL", that is a
+ * wildcard match for any given subcommand.  The first argument is then
+ * replaced with the actual program name to be executed.
  *
  * After checking the acl permissions, the process forks and the child execv's
  * the command with pipes arranged to gather output. The parent waits for the
@@ -200,53 +249,84 @@ fail:
  */
 void
 server_run_command(struct client *client, struct config *config,
-                   struct vector *argv)
+                   struct iovec **argv)
 {
     char *program;
     char *path = NULL;
-    const char *type, *service;
+    char *command = NULL;
+    char *subcommand = NULL;
     struct confline *cline = NULL;
-    int stdout_pipe[2], stderr_pipe[2];
+    int stdin_pipe[2], stdout_pipe[2], stderr_pipe[2];
     char **req_argv = NULL;
-    size_t i;
+    size_t count, i, j, stdin_arg;
     bool ok;
     int fd;
-    struct process process = { 0, { 0, 0 }, -1, 0 };
+    struct process process = { 0, { 0, 0 }, 0, NULL, -1, 0 };
     const char *user = client->user;
 
     /*
      * We need at least one argument.  This is also rejected earlier when
      * parsing the command and checking argc, but may as well be sure.
      */
-    if (argv->count == 0) {
+    if (argv[0] == NULL) {
         notice("empty command from user %s", user);
         server_send_error(client, ERROR_BAD_COMMAND, "Invalid command token");
         goto done;
     }
 
-    /* We refer to these a lot, so give them good aliases. */
-    type = argv->strings[0];
-    service = (argv->count > 1) ? argv->strings[1] : NULL;
+    /*
+     * Neither the command nor the subcommand may ever contain nuls.
+     * Arguments may only contain nuls if they're the argument being passed on
+     * standard input.
+     */
+    for (i = 0; argv[i] != NULL && i < 2; i++) {
+        if (memchr(argv[i]->iov_base, '\0', argv[i]->iov_len)) {
+            notice("%s from user %s contains nul octet",
+                   (i == 0) ? "command" : "subcommand", user);
+            server_send_error(client, ERROR_BAD_COMMAND,
+                              "Invalid command token");
+            goto done;
+        }
+    }
+
+    /* We need the command and subcommand as nul-terminated strings. */
+    command = xstrndup(argv[0]->iov_base, argv[0]->iov_len);
+    if (argv[1] != NULL)
+        subcommand = xstrndup(argv[1]->iov_base, argv[1]->iov_len);
 
     /*
      * Look up the command and the ACL file from the conf file structure in
-     * memory.  Commands with no service argument will only match lines with
-     * the ALL wildcard.
+     * memory.  Commands with no subcommand argument will only match lines
+     * with the ALL wildcard.
      */
     for (i = 0; i < config->count; i++) {
         cline = config->rules[i];
-        if (strcmp(cline->type, type) == 0) {
-            if (strcmp(cline->service, "ALL") == 0
-                || (service != NULL && strcmp(cline->service, service) == 0)) {
+        if (strcmp(cline->command, command) == 0) {
+            if (strcmp(cline->subcommand, "ALL") == 0
+                || (subcommand != NULL
+                    && strcmp(cline->subcommand, subcommand) == 0)) {
                 path = cline->program;
                 break;
             }
         }
     }
 
-    /* From this point on, service is used only for logging. */
-    if (service == NULL)
-        service = "(null)";
+    /*
+     * Arguments may only contain nuls if they're the argument being passed on
+     * standard input.
+     */
+    for (i = 1; argv[i] != NULL; i++) {
+        if ((long) i == cline->stdin_arg)
+            continue;
+        if (argv[i + 1] == NULL && cline->stdin_arg == -1)
+            continue;
+        if (memchr(argv[i]->iov_base, '\0', argv[i]->iov_len)) {
+            notice("argument %d from user %s contains nul octet", i, user);
+            server_send_error(client, ERROR_BAD_COMMAND,
+                              "Invalid command token");
+            goto done;
+        }
+    }
 
     /* Log after we look for command so we can get potentially get logmask. */
     server_log_command(argv, path == NULL ? NULL : cline, user);
@@ -256,33 +336,52 @@ server_run_command(struct client *client, struct config *config,
      * run this command.
      */
     if (path == NULL) {
-        notice("unknown command %s %s from user %s", type, service, user);
+        notice("unknown command %s%s%s from user %s", command,
+               (subcommand == NULL) ? "" : " ",
+               (subcommand == NULL) ? "" : subcommand, user);
         server_send_error(client, ERROR_UNKNOWN_COMMAND, "Unknown command");
         goto done;
     }
     if (!server_config_acl_permit(cline, user)) {
-        notice("access denied: user %s, command %s %s", user, type, service);
+        notice("access denied: user %s, command %s%s%s", user, command,
+               (subcommand == NULL) ? "" : " ",
+               (subcommand == NULL) ? "" : subcommand);
         server_send_error(client, ERROR_ACCESS, "Access denied");
         goto done;
     }
 
-    /* Assemble the argv, envp, and fork off the child to run the command. */
-    req_argv = xmalloc((argv->count + 1) * sizeof(char *));
+    /* Get ready to assemble the argv of the command. */
+    for (count = 0; argv[count] != NULL; count++)
+        ;
+    req_argv = xmalloc((count + 1) * sizeof(char *));
 
     /*
      * Get the real program name, and use it as the first argument in argv
-     * passed to the command.
+     * passed to the command.  Then build the rest of the argv for the
+     * command, splicing out the argument we're passing on stdin (if any).
      */
     program = strrchr(path, '/');
     if (program == NULL)
         program = path;
     else
         program++;
-    req_argv[0] = strdup(program);
-    for (i = 1; i < argv->count; i++) {
-        req_argv[i] = strdup(argv->strings[i]);
+    req_argv[0] = program;
+    if (cline->stdin_arg == -1)
+        stdin_arg = count - 1;
+    else
+        stdin_arg = (size_t) cline->stdin_arg;
+    for (i = 1, j = 1; i < count; i++) {
+        if (i == stdin_arg) {
+            process.input = argv[i];
+            continue;
+        }
+        if (argv[i]->iov_len == 0)
+            req_argv[j] = xstrdup("");
+        else
+            req_argv[j] = xstrndup(argv[i]->iov_base, argv[i]->iov_len);
+        j++;
     }
-    req_argv[i] = NULL;
+    req_argv[j] = NULL;
 
     /*
      * These pipes are used for communication with the child process that 
@@ -293,6 +392,11 @@ server_run_command(struct client *client, struct config *config,
         server_send_error(client, ERROR_INTERNAL, "Internal failure");
         goto done;
     }
+    if (process.input != NULL && pipe(stdin_pipe) != 0) {
+        syswarn("cannot create stdin pipe");
+        server_send_error(client, ERROR_INTERNAL, "Internal failure");
+        goto done;
+    }
 
     /*
      * Flush output before forking, mostly in case -S was given and we've
@@ -317,16 +421,24 @@ server_run_command(struct client *client, struct config *config,
         close(stderr_pipe[1]);
 
         /*
-         * Child doesn't need stdin at all, but just closing it causes
-         * problems for puppet.  Reopen on /dev/null instead.  Ignore failure
-         * here, since it probably won't matter and worst case is that we
-         * leave stdin closed.
+         * Set up stdin pipe if we have input data.
+         *
+         * If we don't have input data, child doesn't need stdin at all, but
+         * just closing it causes problems for puppet.  Reopen on /dev/null
+         * instead.  Ignore failure here, since it probably won't matter and
+         * worst case is that we leave stdin closed.
          */
-        close(0);
-        fd = open("/dev/null", O_RDONLY);
-        if (fd > 0) {
-            dup2(fd, 0);
-            close(fd);
+        if (process.input != NULL) {
+            dup2(stdin_pipe[0], 0);
+            close(stdin_pipe[0]);
+            close(stdin_pipe[1]);
+        } else {
+            close(0);
+            fd = open("/dev/null", O_RDONLY);
+            if (fd > 0) {
+                dup2(fd, 0);
+                close(fd);
+            }
         }
 
         /*
@@ -377,21 +489,34 @@ server_run_command(struct client *client, struct config *config,
     default:
         close(stdout_pipe[1]);
         close(stderr_pipe[1]);
+        if (process.input != NULL)
+            close(stdin_pipe[0]);
 
         /*
-         * Unblock the read ends of the pipes, to enable us to read from both
-         * iteratively.
+         * Unblock the read ends of the output pipes, to enable us to read
+         * from both iteratively, and unblock the write end of the input pipe
+         * if we have one so that we don't block when feeding data to our
+         * child.
          */
         fdflag_nonblocking(stdout_pipe[0], true);
         fdflag_nonblocking(stderr_pipe[0], true);
+        if (process.input != NULL)
+            fdflag_nonblocking(stdin_pipe[1], true);
 
         /*
          * This collects output from both pipes iteratively, while the child
-         * is executing, and processes it.
+         * is executing, and processes it.  It also sends input data if we
+         * have any.
          */
         process.fds[0] = stdout_pipe[0];
         process.fds[1] = stderr_pipe[0];
+        if (process.input != NULL)
+            process.stdin_fd = stdin_pipe[1];
         ok = server_process_output(client, &process);
+        close(process.fds[0]);
+        close(process.fds[1]);
+        if (process.input != NULL)
+            close(process.stdin_fd);
         if (!process.reaped)
             waitpid(process.pid, &process.status, 0);
         if (WIFEXITED(process.status))
@@ -404,17 +529,37 @@ server_run_command(struct client *client, struct config *config,
             else
                 server_v2_send_status(client, process.status);
         }
-        close(process.fds[0]);
-        close(process.fds[1]);
     }
 
  done:
+    if (command != NULL)
+        free(command);
+    if (subcommand != NULL)
+        free(subcommand);
     if (req_argv != NULL) {
-        i = 0;
-        while (req_argv[i] != '\0') {
+        i = 1;
+        while (req_argv[i] != NULL) {
             free(req_argv[i]);
             i++;
         }
         free(req_argv);
     }
 }
+
+
+/*
+ * Free a command, represented as a NULL-terminated array of pointers to iovec
+ * structs.
+ */
+void
+server_free_command(struct iovec **command)
+{
+    struct iovec **arg;
+
+    for (arg = command; *arg != NULL; arg++) {
+        if ((*arg)->iov_base != NULL)
+            free((*arg)->iov_base);
+        free(*arg);
+    }
+    free(command);
+}
