@@ -1,32 +1,21 @@
 /*
  * getnameinfo test suite.
  *
+ * Written by Russ Allbery <rra@stanford.edu>
+ * Copyright 2009 Board of Trustees, Leland Stanford Jr. University
  * Copyright (c) 2004, 2005, 2006, 2007
  *     by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1991, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
  *     2002, 2003 by The Internet Software Consortium and Rich Salz
  *
- * This code is derived from software contributed to the Internet Software
- * Consortium by Rich Salz.
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * See LICENSE for licensing terms.
  */
 
 #include <config.h>
 #include <portable/system.h>
 #include <portable/socket.h>
 
-#include <tests/libtest.h>
+#include <tests/tap/basic.h>
 #include <util/util.h>
 
 int test_getnameinfo(const struct sockaddr *, socklen_t, char *, socklen_t,
@@ -48,7 +37,7 @@ main(void)
     struct servent *serv;
     char *name;
 
-    test_init(26);
+    plan(26);
 
     /*
      * Test the easy stuff that requires no assumptions.  Hopefully everyone
@@ -61,62 +50,62 @@ main(void)
     sin.sin_port = htons(119);
     inet_aton("10.20.30.40", &sin.sin_addr);
     status = test_getnameinfo(sa, sizeof(sin), NULL, 0, NULL, 0, 0);
-    ok_int(1, EAI_NONAME, status);
+    is_int(EAI_NONAME, status, "EAI_NONAME from NULL");
     status = test_getnameinfo(sa, sizeof(sin), node, 0, service, 0, 0);
-    ok_int(2, EAI_NONAME, status);
+    is_int(EAI_NONAME, status, "EAI_NONAME from length of 0");
     status = test_getnameinfo(sa, sizeof(sin), NULL, 0, service,
                               sizeof(service), 0);
-    ok_int(3, 0, status);
+    is_int(0, status, "lookup of port 119");
     serv = getservbyname("nntp", "tcp");
     if (serv == NULL)
-        skip_block(4, 5, "nntp service not found");
+        skip_block(5, "nntp service not found");
     else {
-        ok_string(4, "nntp", service);
+        is_string("nntp", service, "...found nntp");
         service[0] = '\0';
         status = test_getnameinfo(sa, sizeof(sin), NULL, 0, service, 1, 0);
-        ok_int(5, EAI_OVERFLOW, status);
+        is_int(EAI_OVERFLOW, status, "EAI_OVERFLOW with one character");
         status = test_getnameinfo(sa, sizeof(sin), NULL, 0, service, 4, 0);
-        ok_int(6, EAI_OVERFLOW, status);
+        is_int(EAI_OVERFLOW, status, "EAI_OVERFLOW with four characters");
         status = test_getnameinfo(sa, sizeof(sin), NULL, 0, service, 5, 0);
-        ok_int(7, 0, status);
-        ok_string(8, "nntp", service);
+        is_int(0, status, "fits in five characters");
+        is_string("nntp", service, "...and found nntp");
     }
     status = test_getnameinfo(sa, sizeof(sin), NULL, 0, service,
                               sizeof(service), NI_NUMERICSERV);
-    ok_int(9, 0, status);
-    ok_string(10, "119", service);
+    is_int(0, status, "NI_NUMERICSERV");
+    is_string("119", service, "...and returns number");
     sin.sin_port = htons(512);
     status = test_getnameinfo(sa, sizeof(sin), NULL, 0, service,
                               sizeof(service), 0);
-    ok_int(11, 0, status);
+    is_int(0, status, "lookup of 512 TCP");
     serv = getservbyname("exec", "tcp");
     if (serv == NULL)
-        skip(12, "exec service not found");
+        skip("exec service not found");
     else
-        ok_string(12, "exec", service);
+        is_string("exec", service, "...and found exec");
     status = test_getnameinfo(sa, sizeof(sin), NULL, 0, service,
                               sizeof(service), NI_DGRAM);
-    ok_int(13, 0, status);
+    is_int(0, status, "lookup of 512 UDP");
     serv = getservbyname("biff", "udp");
     if (serv == NULL)
-        skip(14, "biff service not found");
+        skip("biff service not found");
     else
-        ok_string(14, "biff", service);
+        is_string("biff", service, "...and found biff");
     status = test_getnameinfo(sa, sizeof(sin), node, sizeof(node), NULL, 0,
                               NI_NUMERICHOST);
-    ok_int(15, 0, status);
-    ok_string(16, "10.20.30.40", node);
+    is_int(0, status, "lookup with NI_NUMERICHOST");
+    is_string("10.20.30.40", node, "...and found correct IP address");
     node[0] = '\0';
     status = test_getnameinfo(sa, sizeof(sin), node, 1, NULL, 0,
                               NI_NUMERICHOST);
-    ok_int(17, EAI_OVERFLOW, status);
+    is_int(EAI_OVERFLOW, status, "EAI_OVERFLOW with one character");
     status = test_getnameinfo(sa, sizeof(sin), node, 11, NULL, 0,
                               NI_NUMERICHOST);
-    ok_int(18, EAI_OVERFLOW, status);
+    is_int(EAI_OVERFLOW, status, "EAI_OVERFLOW with 11 characters");
     status = test_getnameinfo(sa, sizeof(sin), node, 12, NULL, 0,
                               NI_NUMERICHOST);
-    ok_int(19, 0, status);
-    ok_string(20, "10.20.30.40", node);
+    is_int(0, status, "fits into 12 characters");
+    is_string("10.20.30.40", node, "...and found correct IP address");
 
     /*
      * Okay, now it gets annoying.  Do a forward and then reverse lookup of
@@ -125,19 +114,19 @@ main(void)
      */
     hp = gethostbyname("www.isc.org");
     if (hp == NULL)
-        skip_block(21, 2, "cannot look up www.isc.org");
+        skip_block(2, "cannot look up www.isc.org");
     else {
         memcpy(&sin.sin_addr, hp->h_addr, sizeof(sin.sin_addr));
         hp = gethostbyaddr((const void *) &sin.sin_addr, sizeof(sin.sin_addr),
                            AF_INET);
         if (hp == NULL || strchr(hp->h_name, '.') == NULL)
-            skip_block(21, 2, "cannot reverse-lookup www.isc.org");
+            skip_block(2, "cannot reverse-lookup www.isc.org");
         else {
             name = xstrdup(hp->h_name);
             status = test_getnameinfo(sa, sizeof(sin), node, sizeof(node),
                                       NULL, 0, 0);
-            ok_int(21, 0, status);
-            ok_string(22, name, node);
+            is_int(0, status, "lookup of www.isc.org IP address");
+            is_string(name, node, "...matches gethostbyaddr");
             free(name);
         }
     }
@@ -145,16 +134,16 @@ main(void)
     /* Hope that no one is weird enough to put 0.0.0.0 into DNS. */
     inet_aton("0.0.0.0", &sin.sin_addr);
     status = test_getnameinfo(sa, sizeof(sin), node, sizeof(node), NULL, 0, 0);
-    ok_int(23, 0, status);
-    ok_string(24, "0.0.0.0", node);
+    is_int(0, status, "lookup of 0.0.0.0");
+    is_string("0.0.0.0", node, "...returns the IP address");
     node[0] = '\0';
     status = test_getnameinfo(sa, sizeof(sin), node, sizeof(node), NULL, 0,
                               NI_NAMEREQD);
-    ok_int(25, EAI_NONAME, status);
+    is_int(EAI_NONAME, status, "...fails with NI_NAMEREQD");
 
     sin.sin_family = AF_UNIX;
     status = test_getnameinfo(sa, sizeof(sin), node, sizeof(node), NULL, 0, 0);
-    ok_int(26, EAI_FAMILY, status);
+    is_int(EAI_FAMILY, status, "EAI_FAMILY");
 
     return 0;
 }
