@@ -1,32 +1,20 @@
-/* $Id: snprintf-t.c 4022 2008-03-31 06:11:07Z rra $
- *
+/*
  * snprintf test suite.
  *
+ * Written by Russ Allbery <rra@stanford.edu>
+ * Copyright 2009 Board of Trustees, Leland Stanford Jr. University
  * Copyright (c) 2004, 2005, 2006
  *     by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1991, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
  *     2002, 2003 by The Internet Software Consortium and Rich Salz
  *
- * This code is derived from software contributed to the Internet Software
- * Consortium by Rich Salz.
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * See LICENSE for licensing terms.
  */
 
 #include <config.h>
 #include <portable/system.h>
 
-#include <tests/libtest.h>
+#include <tests/tap/basic.h>
 
 int test_snprintf(char *str, size_t count, const char *fmt, ...);
 int test_vsnprintf(char *str, size_t count, const char *fmt, va_list args);
@@ -94,7 +82,7 @@ static unsigned long long ullong_nums[] = {
 
 
 static void
-test_format(int n, int truncate, const char *expected, int count,
+test_format(bool truncate, const char *expected, int count,
             const char *format, ...)
 {
     char buf[128];
@@ -104,16 +92,8 @@ test_format(int n, int truncate, const char *expected, int count,
     va_start(args, format);
     result = test_vsnprintf(buf, truncate ? 32 : sizeof(buf), format, args);
     va_end(args);
-    if (!strcmp(buf, expected) && result == count) {
-        printf("ok %d\n", n);
-    } else {
-        printf("not ok %d\n", n);
-        printf("  format: %s\n", format);
-        if (strcmp(buf, expected))
-            printf("   saw: %s\n  want: %s\n", buf, expected);
-        if (result != count)
-            printf("  %d != %d\n", result, count);
-    }
+    is_string(expected, buf, "format %s, wanted %s", format, expected);
+    is_int(count, result, "...and output length correct");
 }
 
 
@@ -125,75 +105,69 @@ main(void)
     long lcount;
     char lgbuf[128];
 
-    test_init((26 + (ARRAY_SIZE(fp_formats) - 1) * ARRAY_SIZE(fp_nums)
-              + (ARRAY_SIZE(int_formats) - 1) * ARRAY_SIZE(int_nums)
-              + (ARRAY_SIZE(uint_formats) - 1) * ARRAY_SIZE(uint_nums)
-              + (ARRAY_SIZE(llong_formats) - 1) * ARRAY_SIZE(llong_nums)
-              + (ARRAY_SIZE(ullong_formats) - 1) * ARRAY_SIZE(ullong_nums)));
+    plan(8 +
+         (18 + (ARRAY_SIZE(fp_formats) - 1) * ARRAY_SIZE(fp_nums)
+          + (ARRAY_SIZE(int_formats) - 1) * ARRAY_SIZE(int_nums)
+          + (ARRAY_SIZE(uint_formats) - 1) * ARRAY_SIZE(uint_nums)
+          + (ARRAY_SIZE(llong_formats) - 1) * ARRAY_SIZE(llong_nums)
+          + (ARRAY_SIZE(ullong_formats) - 1) * ARRAY_SIZE(ullong_nums)) * 2);
 
-    ok(1, test_snprintf(NULL, 0, "%s", "abcd") == 4);
-    ok(2, test_snprintf(NULL, 0, "%d", 20) == 2);
-    ok(3, test_snprintf(NULL, 0, "Test %.2s", "abcd") == 7);
-    ok(4, test_snprintf(NULL, 0, "%c", 'a') == 1);
-    ok(5, test_snprintf(NULL, 0, "") == 0);
+    is_int(4, test_snprintf(NULL, 0, "%s", "abcd"), "simple string length");
+    is_int(2, test_snprintf(NULL, 0, "%d", 20), "number length");
+    is_int(7, test_snprintf(NULL, 0, "Test %.2s", "abcd"), "limited string");
+    is_int(1, test_snprintf(NULL, 0, "%c", 'a'), "character length");
+    is_int(0, test_snprintf(NULL, 0, ""), "empty format length");
 
-    test_format(6, 1, "abcd", 4, "%s", "abcd");
-    test_format(7, 1, "20", 2, "%d", 20);
-    test_format(8, 1, "Test ab", 7, "Test %.2s", "abcd");
-    test_format(9, 1, "a", 1, "%c", 'a');
-    test_format(10, 1, "", 0, "");
-    test_format(11, 1, "abcdefghijklmnopqrstuvwxyz01234", 36, "%s",
-                string);
-    test_format(12, 1, "abcdefghij", 10, "%.10s", string);
-    test_format(13, 1, "  abcdefghij", 12, "%12.10s", string);
-    test_format(14, 1, "    abcdefghijklmnopqrstuvwxyz0", 40, "%40s",
-                string);
-    test_format(15, 1, "abcdefghij    ", 14, "%-14.10s", string);
-    test_format(16, 1, "              abcdefghijklmnopq", 50, "%50s",
-                string);
-    test_format(17, 1, "%abcd%", 6, "%%%0s%%", "abcd");
-    test_format(18, 1, "", 0, "%.0s", string);
-    test_format(19, 1, "abcdefghijklmnopqrstuvwxyz  444", 32, "%.26s  %d",
+    test_format(true, "abcd", 4, "%s", "abcd");
+    test_format(true, "20", 2, "%d", 20);
+    test_format(true, "Test ab", 7, "Test %.2s", "abcd");
+    test_format(true, "a", 1, "%c", 'a');
+    test_format(true, "", 0, "");
+    test_format(true, "abcdefghijklmnopqrstuvwxyz01234", 36, "%s", string);
+    test_format(true, "abcdefghij", 10, "%.10s", string);
+    test_format(true, "  abcdefghij", 12, "%12.10s", string);
+    test_format(true, "    abcdefghijklmnopqrstuvwxyz0", 40, "%40s", string);
+    test_format(true, "abcdefghij    ", 14, "%-14.10s", string);
+    test_format(true, "              abcdefghijklmnopq", 50, "%50s", string);
+    test_format(true, "%abcd%", 6, "%%%0s%%", "abcd");
+    test_format(true, "", 0, "%.0s", string);
+    test_format(true, "abcdefghijklmnopqrstuvwxyz  444", 32, "%.26s  %d",
                 string, 4444);
-    test_format(20, 1, "abcdefghijklmnopqrstuvwxyz  -2.", 32,
-                "%.26s  %.1f", string, -2.5);
-    test_format(21, 1, "abcdefghij4444", 14, "%.10s%n%d", string, &count,
-                4444);
-    ok(22, count == 10);
-    test_format(23, 1, "abcdefghijklmnopqrstuvwxyz01234", 36, "%n%s%ln",
+    test_format(true, "abcdefghijklmnopqrstuvwxyz  -2.", 32, "%.26s  %.1f",
+                string, -2.5);
+    test_format(true, "abcdefghij4444", 14, "%.10s%n%d", string, &count, 4444);
+    is_int(10, count, "correct output from %%n");
+    test_format(true, "abcdefghijklmnopqrstuvwxyz01234", 36, "%n%s%ln",
                 &count, string, &lcount);
-    ok(24, count == 0);
-    ok(25, lcount == 31);
-    test_format(26, 1, "(null)", 6, "%s", NULL);
+    is_int(0, count, "correct output from two %%n");
+    is_int(31, lcount, "correct output from long %%ln");
+    test_format(true, "(null)", 6, "%s", NULL);
 
     n = 26;
     for (i = 0; fp_formats[i] != NULL; i++)
         for (j = 0; j < ARRAY_SIZE(fp_nums); j++) {
             count = sprintf(lgbuf, fp_formats[i], fp_nums[j]);
-            test_format(++n, 0, lgbuf, count, fp_formats[i], fp_nums[j]);
+            test_format(false, lgbuf, count, fp_formats[i], fp_nums[j]);
         }
     for (i = 0; int_formats[i] != NULL; i++)
         for (j = 0; j < ARRAY_SIZE(int_nums); j++) {
             count = sprintf(lgbuf, int_formats[i], int_nums[j]);
-            test_format(++n, 0, lgbuf, count, int_formats[i],
-                        int_nums[j]);
+            test_format(false, lgbuf, count, int_formats[i], int_nums[j]);
         }
     for (i = 0; uint_formats[i] != NULL; i++)
         for (j = 0; j < ARRAY_SIZE(uint_nums); j++) {
             count = sprintf(lgbuf, uint_formats[i], uint_nums[j]);
-            test_format(++n, 0, lgbuf, count, uint_formats[i],
-                        uint_nums[j]);
+            test_format(false, lgbuf, count, uint_formats[i], uint_nums[j]);
         }
     for (i = 0; llong_formats[i] != NULL; i++)
         for (j = 0; j < ARRAY_SIZE(llong_nums); j++) {
             count = sprintf(lgbuf, llong_formats[i], llong_nums[j]);
-            test_format(++n, 0, lgbuf, count, llong_formats[i],
-                        llong_nums[j]);
+            test_format(false, lgbuf, count, llong_formats[i], llong_nums[j]);
         }
     for (i = 0; ullong_formats[i] != NULL; i++)
         for (j = 0; j < ARRAY_SIZE(ullong_nums); j++) {
             count = sprintf(lgbuf, ullong_formats[i], ullong_nums[j]);
-            test_format(++n, 0, lgbuf, count, ullong_formats[i],
+            test_format(false, lgbuf, count, ullong_formats[i],
                         ullong_nums[j]);
         }
 
