@@ -2,7 +2,8 @@
  * Test suite for errors returned by the server.
  *
  * Written by Russ Allbery <rra@stanford.edu>
- * Copyright 2006, 2007 Board of Trustees, Leland Stanford Jr. University
+ * Copyright 2006, 2007, 2009
+ *     Board of Trustees, Leland Stanford Jr. University
  *
  * See LICENSE for licensing terms.
  */
@@ -15,7 +16,9 @@
 
 #include <client/internal.h>
 #include <client/remctl.h>
-#include <tests/libtest.h>
+#include <tests/tap/basic.h>
+#include <tests/tap/kerberos.h>
+#include <tests/tap/remctl.h>
 #include <util/util.h>
 
 
@@ -32,23 +35,23 @@ test_error(struct remctl *r, const char *arg)
 
     command[1] = arg;
     if (!remctl_command(r, command)) {
-        warn("remctl error %s", remctl_error(r));
+        notice("# remctl error %s", remctl_error(r));
         return ERROR_INTERNAL;
     }
     do {
         output = remctl_output(r);
         switch (output->type) {
         case REMCTL_OUT_OUTPUT:
-            warn("test %s returned output: %.*s", arg, (int) output->length,
-                 output->data);
+            notice("# test %s returned output: %.*s", arg,
+                   (int) output->length, output->data);
             break;
         case REMCTL_OUT_STATUS:
-            warn("test %s returned status %d", arg, output->status);
+            notice("# test %s returned status %d", arg, output->status);
             return ERROR_INTERNAL;
         case REMCTL_OUT_ERROR:
             return output->error;
         case REMCTL_OUT_DONE:
-            warn("unexpected done token");
+            notice("# unexpected done token");
             return ERROR_INTERNAL;
         }
     } while (output->type == REMCTL_OUT_OUTPUT);
@@ -74,7 +77,7 @@ test_excess_args(struct remctl *r)
         command[i] = "a";
     command[10 * 1024 + 2] = NULL;
     if (!remctl_command(r, command)) {
-        warn("remctl error %s", remctl_error(r));
+        notice("# remctl error %s", remctl_error(r));
         return ERROR_INTERNAL;
     }
     free(command);
@@ -82,16 +85,16 @@ test_excess_args(struct remctl *r)
         output = remctl_output(r);
         switch (output->type) {
         case REMCTL_OUT_OUTPUT:
-            warn("test echo returned output: %.*s", (int) output->length,
-                 output->data);
+            notice("# test echo returned output: %.*s", (int) output->length,
+                   output->data);
             break;
         case REMCTL_OUT_STATUS:
-            warn("test echo returned status %d", output->status);
+            notice("# test echo returned status %d", output->status);
             return ERROR_INTERNAL;
         case REMCTL_OUT_ERROR:
             return output->error;
         case REMCTL_OUT_DONE:
-            warn("unexpected done token");
+            notice("# unexpected done token");
             return ERROR_INTERNAL;
         }
     } while (output->type == REMCTL_OUT_OUTPUT);
@@ -102,42 +105,37 @@ test_excess_args(struct remctl *r)
 int
 main(void)
 {
-    char *principal;
+    char *principal, *config, *path;
     struct remctl *r;
     pid_t remctld;
     int status;
 
-    test_init(4);
-
     /* Unless we have Kerberos available, we can't really do anything. */
+    if (chdir(getenv("SOURCE")) < 0)
+        bail("can't chdir to SOURCE");
     principal = kerberos_setup();
-    if (principal == NULL) {
-        skip_block(1, 4, "Kerberos tests not configured");
-        return 0;
-    }
-
-    /* Spawn the remctld server. */
-    remctld = spawn_remctld(principal);
-    if (remctld <= 0)
-        die("cannot spawn remctld");
+    if (principal == NULL)
+        skip_all("Kerberos tests not configured");
+    plan(4);
+    config = concatpath(getenv("SOURCE"), "data/conf-simple");
+    path = concatpath(getenv("BUILD"), "../server/remctld");
+    remctld = remctld_start(path, principal, config);
 
     /* Run the tests. */
     r = remctl_new();
-    if (!remctl_open(r, "localhost", 14444, principal))
-        die("cannot contact remctld");
+    if (!remctl_open(r, "localhost", 14373, principal))
+        bail("cannot contact remctld");
     status = test_error(r, "bad-command");
-    ok_int(1, ERROR_UNKNOWN_COMMAND, status);
+    is_int(ERROR_UNKNOWN_COMMAND, status, "unknown command");
     status = test_error(r, "noauth");
-    ok_int(2, ERROR_ACCESS, status);
+    is_int(ERROR_ACCESS, status, "access denied");
     status = test_excess_args(r);
-    ok_int(3, ERROR_TOOMANY_ARGS, status);
+    is_int(ERROR_TOOMANY_ARGS, status, "too many arguments");
     status = test_error(r, NULL);
-    ok_int(4, ERROR_UNKNOWN_COMMAND, status);
+    is_int(ERROR_UNKNOWN_COMMAND, status, "unknown command");
     remctl_close(r);
 
-    kill(remctld, SIGTERM);
-    waitpid(remctld, NULL, 0);
-    unlink("data/pid");
-
+    remctld_stop(remctld);
+    kerberos_cleanup();
     return 0;
 }
