@@ -2,7 +2,8 @@
  * Test suite for version negotiation in the server.
  *
  * Written by Russ Allbery <rra@stanford.edu>
- * Copyright 2006, 2007 Board of Trustees, Leland Stanford Jr. University
+ * Copyright 2006, 2007, 2009
+ *     Board of Trustees, Leland Stanford Jr. University
  *
  * See LICENSE for licensing terms.
  */
@@ -16,7 +17,9 @@
 
 #include <client/internal.h>
 #include <client/remctl.h>
-#include <tests/libtest.h>
+#include <tests/tap/basic.h>
+#include <tests/tap/kerberos.h>
+#include <tests/tap/remctl.h>
 #include <util/util.h>
 
 /* A command token to run test test. */
@@ -31,31 +34,28 @@ static const char token[] = {
 int
 main(void)
 {
-    char *principal;
+    char *principal, *config, *path;
     struct remctl *r;
     pid_t remctld;
     OM_uint32 major, minor;
     int flags, status;
     gss_buffer_desc tok;
 
-    test_init(8);
-
     /* Unless we have Kerberos available, we can't really do anything. */
+    if (chdir(getenv("SOURCE")) < 0)
+        bail("can't chdir to SOURCE");
     principal = kerberos_setup();
-    if (principal == NULL) {
-        skip_block(1, 8, "Kerberos tests not configured");
-        return 0;
-    }
-
-    /* Spawn the remctld server. */
-    remctld = spawn_remctld(principal);
-    if (remctld <= 0)
-        die("cannot spawn remctld");
+    if (principal == NULL)
+        skip_all("Kerberos tests not configured");
+    plan(8);
+    config = concatpath(getenv("SOURCE"), "data/conf-simple");
+    path = concatpath(getenv("BUILD"), "../server/remctld");
+    remctld = remctld_start(path, principal, config);
 
     /* Open the connection to the site. */
     r = remctl_new();
-    ok(1, r != NULL);
-    ok(2, remctl_open(r, "localhost", 14444, principal));
+    ok(r != NULL, "remctl_new");
+    ok(remctl_open(r, "localhost", 14373, principal), "remctl_open");
 
     /* Send the command token. */
     tok.length = sizeof(token);
@@ -63,23 +63,22 @@ main(void)
     status = token_send_priv(r->fd, r->context, TOKEN_DATA | TOKEN_PROTOCOL,
                              &tok, &major, &minor);
     if (status != TOKEN_OK)
-        die("cannot send token");
+        bail("cannot send token");
 
     /* Accept the remote token. */
     status = token_recv_priv(r->fd, r->context, &flags, &tok, 1024 * 64,
                              &major, &minor);
-    ok_int(3, TOKEN_OK, status);
-    ok_int(4, TOKEN_DATA | TOKEN_PROTOCOL, flags);
-    ok_int(5, 3, tok.length);
-    ok_int(6, 2, ((char *) tok.value)[0]);
-    ok_int(7, MESSAGE_VERSION, ((char *) tok.value)[1]);
-    ok_int(8, 2, ((char *) tok.value)[0]);
+    is_int(TOKEN_OK, status, "received token correctly");
+    is_int(TOKEN_DATA | TOKEN_PROTOCOL, flags, "token had correct flags");
+    is_int(3, tok.length, "token had correct length");
+    is_int(2, ((char *) tok.value)[0], "protocol version is 2");
+    is_int(MESSAGE_VERSION, ((char *) tok.value)[1], "message version code");
+    is_int(2, ((char *) tok.value)[2], "highest supported version is 2");
 
     /* Close things out. */
     remctl_close(r);
-    kill(remctld, SIGTERM);
-    waitpid(remctld, NULL, 0);
-    unlink("data/pid");
 
+    remctld_stop(remctld);
+    kerberos_cleanup();
     return 0;
 }
