@@ -2,7 +2,7 @@
  * fdflag test suite.
  *
  * Written by Russ Allbery <rra@stanford.edu>
- * Copyright 2008 Board of Trustees, Leland Stanford Jr. University
+ * Copyright 2008, 2009 Board of Trustees, Leland Stanford Jr. University
  *
  * See LICENSE for licensing terms.
  */
@@ -14,7 +14,7 @@
 #include <errno.h>
 #include <sys/wait.h>
 
-#include <tests/libtest.h>
+#include <tests/tap/basic.h>
 #include <util/util.h>
 
 
@@ -28,32 +28,32 @@ main(void)
     pid_t child;
     char buffer[] = "D";
 
-    test_init(8);
+    plan(8);
 
     /* Parent will create the socket first to get the port number. */
     memset(&sin, '\0', sizeof(sin));
     sin.sin_family = AF_INET;
     master = socket(AF_INET, SOCK_STREAM, 0);
     if (master == -1)
-        sysdie("socket creation failed");
+        sysbail("socket creation failed");
     if (bind(master, (struct sockaddr *) &sin, sizeof(sin)) < 0)
-        sysdie("bind failed");
+        sysbail("bind failed");
     size = sizeof(sin);
     if (getsockname(master, (struct sockaddr *) &sin, &size) < 0)
-        sysdie("getsockname failed");
+        sysbail("getsockname failed");
     if (listen(master, 1) < 0)
-        sysdie("listen failed");
+        sysbail("listen failed");
 
     /* Duplicate standard output to test close-on-exec. */
     out1 = 8;
     out2 = 9;
     if (dup2(fileno(stdout), out1) < 0)
-        sysdie("cannot dup stdout to fd 8");
+        sysbail("cannot dup stdout to fd 8");
     if (dup2(fileno(stdout), out2) < 0)
-        sysdie("cannot dup stdout to fd 9");
-    ok(1, fdflag_close_exec(out1, true));
-    ok(2, fdflag_close_exec(out2, true));
-    ok(3, fdflag_close_exec(out2, false));
+        sysbail("cannot dup stdout to fd 9");
+    ok(fdflag_close_exec(out1, true), "set fd 8 to close-on-exec");
+    ok(fdflag_close_exec(out2, true), "set fd 9 to close-on-exec");
+    ok(fdflag_close_exec(out2, false), "set fd 9 back to regular");
 
     /*
      * Fork, child closes the open socket and then tries to connect, parent
@@ -67,30 +67,30 @@ main(void)
      */
     child = fork();
     if (child < 0) {
-        sysdie("fork failed");
+        sysbail("fork failed");
     } else if (child != 0) {
         size = sizeof(sin);
         data = accept(master, (struct sockaddr *) &sin, &size);
         close(master);
         if (data < 0)
-            sysdie("accept failed");
-        ok(4, fdflag_nonblocking(data, true));
+            sysbail("accept failed");
+        ok(fdflag_nonblocking(data, true), "set socket non-blocking");
         status = read(data, buffer, sizeof(buffer));
-        ok_int(5, -1, status);
-        ok_int(6, EAGAIN, errno);
+        is_int(-1, status, "got -1 from non-blocking read");
+        is_int(EAGAIN, errno, "...with EAGAIN errno");
         write(data, buffer, sizeof(buffer));
         close(data);
     } else {
         data = socket(AF_INET, SOCK_STREAM, 0);
         if (data < 0)
-            sysdie("child socket failed");
+            sysbail("child socket failed");
         if (connect(data, (struct sockaddr *) &sin, sizeof(sin)) < 0)
-            sysdie("child connect failed");
+            sysbail("child connect failed");
         read(data, buffer, sizeof(buffer));
         fclose(stderr);
         execlp("sh", "sh", "-c",
                "printf 'not ' >&8; echo ok 7; echo 'ok 8' >&9", (char *) 0);
-        sysdie("exec failed");
+        sysbail("exec failed");
     }
     waitpid(child, NULL, 0);
     exit(0);
