@@ -2,7 +2,8 @@
  * gss-tokens test suite.
  *
  * Written by Russ Allbery <rra@stanford.edu>
- * Copyright 2006, 2007 Board of Trustees, Leland Stanford Jr. University
+ * Copyright 2006, 2007, 2009
+ *     Board of Trustees, Leland Stanford Jr. University
  *
  * See LICENSE for licensing terms.
  */
@@ -11,7 +12,8 @@
 #include <portable/system.h>
 #include <portable/gssapi.h>
 
-#include <tests/libtest.h>
+#include <tests/tap/basic.h>
+#include <tests/tap/kerberos.h>
 #include <util/util.h>
 
 extern char send_buffer[2048];
@@ -33,14 +35,11 @@ main(void)
     gss_OID doid;
     int status, flags;
 
-    test_init(26);
-
     /* Unless we have Kerberos available, we can't really do anything. */
     principal = kerberos_setup();
-    if (principal == NULL) {
-        skip_block(1, 26, "Kerberos tests not configured");
-        return 0;
-    }
+    if (principal == NULL)
+        skip_all("Kerberos tests not configured");
+    plan(26);
 
     /*
      * We have to set up a context first in order to do this test, which is
@@ -51,7 +50,7 @@ main(void)
     s_stat = gss_import_name(&s_min_stat, &name_buf, GSS_C_NT_USER_NAME,
                              &server_name);
     if (s_stat != GSS_S_COMPLETE)
-        die("cannot import name");
+        bail("cannot import name");
     server_ctx = GSS_C_NO_CONTEXT;
     client_ctx = GSS_C_NO_CONTEXT;
     token_ptr = GSS_C_NO_BUFFER;
@@ -78,23 +77,23 @@ main(void)
     } while (c_stat == GSS_S_CONTINUE_NEEDED
              || s_stat == GSS_S_CONTINUE_NEEDED);
     if (c_stat != GSS_S_COMPLETE || s_stat != GSS_S_COMPLETE)
-        die("cannot establish context");
+        bail("cannot establish context");
 
     /* Okay, we should now be able to send and receive a token. */
     server_tok.value = (char *) "hello";
     server_tok.length = 5;
     status = token_send_priv(0, server_ctx, 3, &server_tok, &s_stat,
                              &s_min_stat);
-    ok_int(1, TOKEN_OK, status);
-    ok_int(2, 3, send_flags);
-    ok(3, send_length > 5);
+    is_int(TOKEN_OK, status, "sent a token successfully");
+    is_int(3, send_flags, "...with the right flags");
+    ok(send_length > 5, "...and enough length");
     server_tok.value = send_buffer;
     server_tok.length = send_length;
     c_stat = gss_unwrap(&c_min_stat, client_ctx, &server_tok, &client_tok,
                         NULL, NULL);
-    ok_int(4, GSS_S_COMPLETE, c_stat);
-    ok_int(5, 5, client_tok.length);
-    ok(6, memcmp(client_tok.value, "hello", 5) == 0);
+    is_int(GSS_S_COMPLETE, c_stat, "...and it unwrapped");
+    is_int(5, client_tok.length, "...with the right length");
+    ok(memcmp(client_tok.value, "hello", 5) == 0, "...and contents");
     gss_release_buffer(&c_min_stat, &client_tok);
     client_tok.length = 0;
     client_tok.value = NULL;
@@ -102,28 +101,28 @@ main(void)
     server_tok.length = 5;
     status = token_send_priv(0, server_ctx, 3, &server_tok, &s_stat,
                              &s_min_stat);
-    ok_int(7, TOKEN_OK, status);
+    is_int(TOKEN_OK, status, "sent another token");
     memcpy(recv_buffer, send_buffer, send_length);
     recv_length = send_length;
     recv_flags = send_flags;
     status = token_recv_priv(0, client_ctx, &flags, &client_tok, 1024,
                              &s_stat, &c_min_stat);
-    ok_int(8, TOKEN_OK, status);
-    ok_int(9, 5, client_tok.length);
-    ok(10, memcmp(client_tok.value, "hello", 5) == 0);
-    ok_int(11, 3, flags);
+    is_int(TOKEN_OK, status, "received the token");
+    is_int(5, client_tok.length, "...with the right length");
+    ok(memcmp(client_tok.value, "hello", 5) == 0, "...and the right data");
+    is_int(3, flags, "...and the right flags");
     gss_release_buffer(&c_min_stat, &client_tok);
 
     /* Test receiving too large of a token. */
     status = token_recv_priv(0, client_ctx, &flags, &client_tok, 4, &s_stat,
                              &s_min_stat);
-    ok_int(12, TOKEN_FAIL_LARGE, status);
+    is_int(TOKEN_FAIL_LARGE, status, "receiving too large of a token");
 
     /* Test receiving a corrupt token. */
     recv_length = 4;
     status = token_recv_priv(0, client_ctx, &flags, &client_tok, 1024,
                              &s_stat, &s_min_stat);
-    ok_int(13, TOKEN_FAIL_GSSAPI, status);
+    is_int(TOKEN_FAIL_GSSAPI, status, "receiving a corrupt token");
 
     /*
      * Now, fake up a token to make sure that token_recv_priv is doing the
@@ -134,16 +133,16 @@ main(void)
     client_tok.length = 5;
     c_stat = gss_wrap(&c_min_stat, client_ctx, 1, GSS_C_QOP_DEFAULT,
                       &client_tok, NULL, &server_tok);
-    ok_int(14, GSS_S_COMPLETE, c_stat);
+    is_int(GSS_S_COMPLETE, c_stat, "wrapped a fake token");
     recv_length = server_tok.length;
     memcpy(recv_buffer, server_tok.value, server_tok.length);
     gss_release_buffer(&c_min_stat, &server_tok);
     status = token_recv_priv(0, server_ctx, &flags, &server_tok, 1024,
                              &s_stat, &s_min_stat);
-    ok_int(15, TOKEN_OK, status);
-    ok_int(16, 5, flags);
-    ok_int(17, 5, server_tok.length);
-    ok(18, memcmp(server_tok.value, "hello", 5) == 0);
+    is_int(TOKEN_OK, status, "received a fake token");
+    is_int(5, flags, "...with the right flags");
+    is_int(5, server_tok.length, "...and the right length");
+    ok(memcmp(server_tok.value, "hello", 5) == 0, "...and data");
     gss_release_buffer(&c_min_stat, &server_tok);
 
     /* Test the stupid protocol v1 MIC stuff. */
@@ -151,28 +150,28 @@ main(void)
     server_tok.length = 5;
     c_stat = gss_get_mic(&c_min_stat, client_ctx, GSS_C_QOP_DEFAULT,
                          &server_tok, &client_tok);
-    ok_int(19, GSS_S_COMPLETE, c_stat);
+    is_int(GSS_S_COMPLETE, c_stat, "got MIC for protocol v1 token");
     memcpy(recv_buffer, client_tok.value, client_tok.length);
     recv_length = client_tok.length;
     recv_flags = TOKEN_MIC;
     status = token_send_priv(0, server_ctx, TOKEN_DATA | TOKEN_SEND_MIC,
                              &server_tok, &s_stat, &s_min_stat);
-    ok_int(20, TOKEN_OK, status);
+    is_int(TOKEN_OK, status, "sent protocol v1 token with MIC");
     memcpy(recv_buffer, send_buffer, send_length);
     recv_length = send_length;
     recv_flags = send_flags;
     status = token_recv_priv(0, client_ctx, &flags, &client_tok, 1024,
                              &c_stat, &c_min_stat);
-    ok_int(21, TOKEN_OK, status);
-    ok_int(22, TOKEN_DATA, flags);
-    ok_int(23, 5, client_tok.length);
-    ok(24, memcmp(client_tok.value, "hello", 5) == 0);
-    ok_int(25, TOKEN_MIC, send_flags);
+    is_int(TOKEN_OK, status, "received protocol v1 token with MIC");
+    is_int(TOKEN_DATA, flags, "...with the right flags");
+    is_int(5, client_tok.length, "...and the right length");
+    ok(memcmp(client_tok.value, "hello", 5) == 0, "...and the right data");
+    is_int(TOKEN_MIC, send_flags, "...and the right send flags");
     server_tok.value = send_buffer;
     server_tok.length = send_length;
     s_stat = gss_verify_mic(&s_min_stat, server_ctx, &client_tok, &server_tok,
                             NULL);
-    ok_int(26, GSS_S_COMPLETE, s_stat);
+    is_int(GSS_S_COMPLETE, s_stat, "...and would send correct MIC");
 
     return 0;
 }
