@@ -2,7 +2,8 @@
  * tokens test suite.
  *
  * Written by Russ Allbery <rra@stanford.edu>
- * Copyright 2006, 2007 Board of Trustees, Leland Stanford Jr. University
+ * Copyright 2006, 2007, 2009
+ *     Board of Trustees, Leland Stanford Jr. University
  *
  * See LICENSE for licensing terms.
  */
@@ -17,7 +18,7 @@
 #include <sys/time.h>
 #include <sys/wait.h>
 
-#include <tests/libtest.h>
+#include <tests/tap/basic.h>
 #include <util/util.h>
 
 /* A token for testing. */
@@ -37,21 +38,21 @@ create_server(void)
 
     saddr.sin_family = AF_INET;
     saddr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
-    saddr.sin_port = htons(14444);
+    saddr.sin_port = htons(14373);
     fd = socket(AF_INET, SOCK_STREAM, 0);
     if (fd < 0)
-        sysdie("error creating socket");
+        sysbail("error creating socket");
     setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char *) &on, sizeof(on));
     if (bind(fd, (struct sockaddr *) &saddr, sizeof(saddr)) < 0)
-        sysdie("error binding socket");
+        sysbail("error binding socket");
     if (listen(fd, 1) < 0)
-        sysdie("error listening on socket");
+        sysbail("error listening on socket");
     marker = open("server-ready", O_CREAT | O_TRUNC, 0666);
     if (marker < 0)
-        sysdie("cannot create marker file");
+        sysbail("cannot create marker file");
     conn = accept(fd, NULL, 0);
     if (conn < 0)
-        sysdie("error accepting connection");
+        sysbail("error accepting connection");
     return conn;
 }
 
@@ -69,10 +70,10 @@ create_client(void)
 
     saddr.sin_family = AF_INET;
     saddr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
-    saddr.sin_port = htons(14444);
+    saddr.sin_port = htons(14373);
     fd = socket(AF_INET, SOCK_STREAM, 0);
     if (fd < 0)
-        sysdie("error creating socket");
+        sysbail("error creating socket");
     alarm(1);
     while (access("server-ready", F_OK) != 0) {
         tv.tv_sec = 0;
@@ -81,7 +82,7 @@ create_client(void)
     }
     alarm(0);
     if (connect(fd, (struct sockaddr *) &saddr, sizeof(saddr)) < 0)
-        sysdie("error connecting");
+        sysbail("error connecting");
     return fd;
 }
 
@@ -122,12 +123,14 @@ main(void)
 
     alarm(2);
 
-    test_init(10);
+    plan(10);
+    if (chdir(getenv("BUILD")) < 0)
+        sysbail("can't chdir to BUILD");
 
     unlink("server-ready");
     child = fork();
     if (child < 0)
-        sysdie("cannot fork");
+        sysbail("cannot fork");
     else if (child == 0) {
         server = create_server();
         send_regular_token(server);
@@ -135,15 +138,15 @@ main(void)
     } else {
         client = create_client();
         length = read(client, buffer, 12);
-        ok_int(1, 10, length);
-        ok(2, memcmp(buffer, token, 10) == 0);
+        is_int(10, length, "received token has correct length");
+        ok(memcmp(buffer, token, 10) == 0, "...and correct data");
         waitpid(child, NULL, 0);
     }
 
     unlink("server-ready");
     child = fork();
     if (child < 0)
-        sysdie("cannot fork");
+        sysbail("cannot fork");
     else if (child == 0) {
         server = create_server();
         send_hand_token(server);
@@ -151,17 +154,17 @@ main(void)
     } else {
         client = create_client();
         status = token_recv(client, &flags, &result, 5);
-        ok_int(3, TOKEN_OK, status);
-        ok_int(4, 3, flags);
-        ok_int(5, 5, result.length);
-        ok(6, memcmp(result.value, "hello", 5) == 0);
+        is_int(TOKEN_OK, status, "received hand-rolled token");
+        is_int(3, flags, "...with right flags");
+        is_int(5, result.length, "...and right length");
+        ok(memcmp(result.value, "hello", 5) == 0, "...and right data");
         waitpid(child, NULL, 0);
     }
 
     unlink("server-ready");
     child = fork();
     if (child < 0)
-        sysdie("cannot fork");
+        sysbail("cannot fork");
     else if (child == 0) {
         server = create_server();
         xwrite(server, "\0\0\0\0\1", 5);
@@ -169,14 +172,14 @@ main(void)
     } else {
         client = create_client();
         status = token_recv(client, &flags, &result, 200);
-        ok_int(7, TOKEN_FAIL_INVALID, status);
+        is_int(TOKEN_FAIL_INVALID, status, "receive invalid token");
         waitpid(child, NULL, 0);
     }
 
     unlink("server-ready");
     child = fork();
     if (child < 0)
-        sysdie("cannot fork");
+        sysbail("cannot fork");
     else if (child == 0) {
         server = create_server();
         send_hand_token(server);
@@ -184,14 +187,14 @@ main(void)
     } else {
         client = create_client();
         status = token_recv(client, &flags, &result, 4);
-        ok_int(8, TOKEN_FAIL_LARGE, status);
+        is_int(TOKEN_FAIL_LARGE, status, "receive too-large token");
         waitpid(child, NULL, 0);
     }
 
     unlink("server-ready");
     child = fork();
     if (child < 0)
-        sysdie("cannot fork");
+        sysbail("cannot fork");
     else if (child == 0) {
         server = create_server();
         close(server);
@@ -199,7 +202,7 @@ main(void)
     } else {
         client = create_client();
         status = token_recv(client, &flags, &result, 4);
-        ok_int(9, TOKEN_FAIL_EOF, status);
+        is_int(TOKEN_FAIL_EOF, status, "receive end of file");
         waitpid(child, NULL, 0);
     }
     unlink("server-ready");
@@ -207,14 +210,14 @@ main(void)
     /* Special test for error handling when sending tokens. */
     server = open("/dev/full", O_RDWR);
     if (server < 0)
-        skip(10, "/dev/full not available");
+        skip("/dev/full not available");
     else {
         result.value = xmalloc(5);
         memcpy(result.value, "hello", 5);
         result.length = 5;
         status = token_send(server, 3, &result);
         free(result.value);
-        ok_int(10, TOKEN_FAIL_SOCKET, status);
+        is_int(TOKEN_FAIL_SOCKET, status, "can't send due to system error");
         close(server);
     }
 
